
* EXAMPLE 1
select three variables from data set ADMITS
;

data selvar1 ;
set ex.admits (keep = pt_id admdate disdate);
run;

title 'SELVAR1';
proc print data=selvar1;
run;

proc sql;
create table selvar2 as
select pt_id, admdate, disdate
from ex.admits ;
quit;

title 'SELVAR2';
proc print data=selvar2;
run;

* EXTRA
no new data set - print directly from ex admits
;

title 'PRINT SELECTED VARIABLES DIRECTLY FROM EX.ADMITS - PROC PRINT';
proc print data=ex.admits;
var pt_id admdate disdate;
run;

title 'PRINT SELECTED VARIABLES DIRECTLY FROM EX.ADMITS - PROC SQL';
proc sql;
select pt_id, admdate, disdate
from ex.admits ;
quit;

* EXAMPLE 2A
select observations from data set ADMITS
based on the value of an already existing variable (HOSP)
;

data vahosp1 ;
set ex.admits (where=(hosp eq 3));
run;

title 'VAHOSP1';
proc print data=vahosp1;
run;

proc sql feedback;
create table vahosp2 as
select *
from ex.admits
where hosp eq 3;
quit;

title 'VAHOSP2';
proc print data=vahosp2;
run;
       
* EXAMPLE 2B
create a new variable (DXGRP)
;

data mi1 ;
set ex.admits;
length dxgrp $5;
if primdx eq: '410' then dxgrp = 'MI';
else
if primdx eq: '428' then dxgrp = 'CHF';
else                     dxgrp = 'OTHER';
run;

title 'MI1';
proc print data=mi1;
run;

* EXTRA
use a SELECT statement instead of IF-THEN-ELSE
;

data mi1s;
set ex.admits;
length dxgrp $5;
select;
   when (primdx eq: '410') dxgrp = 'MI';
   when (primdx eq: '428') dxgrp = 'CHF';
   otherwise               dxgrp = 'OTHER';
end;
run;

title 'MI1S';
proc print data=mi1s;
run;

proc sql;
create table mi2 as
select *,
       case
	    when primdx like '410%' then 'MI'
	    when primdx like '428%' then 'CHF'
	    else                         'OTHER' 
	  end as dxgrp
from ex.admits
quit;

title 'MI2';
proc print data=mi2;
run;

* EXAMPLE 2C
select observations from data set ADMITS
based on the vaue of a newly created variable (LOS)
;

data twowks1 ;
set ex.admits (keep = pt_id hosp admdate disdate) ;
los = (disdate - admdate) + 1;
attrib los length=4 label='length of stay';
if los ge 14;
run;

title 'TWOWKS1';
proc print data=twowks1;
run;

proc sql;
create table twowks2 as
select pt_id, hosp, admdate, disdate, 
       (disdate - admdate) + 1 as los length=4 label='length of stay'
from ex.admits
where calculated los ge 14;
quit;

title 'TWOWKS2';
proc print data=twowks2 label;
run;

* EXTRA
no name used for the new variable
;

proc sql;
create table twowks2 as
select pt_id, hosp, admdate, disdate,  (disdate - admdate) + 1
from ex.admits
where (disdate - admdate) + 1 ge 14;
quit;

title 'TWOWKS2';
proc print data=twowks2;
run;

* EXAMPLE 2D
select observations from one data set 
based on the value of a variable in another data set (HOSPNAME, VA hospitals) 
;

proc sort data = ex.admits out=admits;
by hosp ;
run;

data vahosp1d (drop = hospname) ;
merge admits      (in=adm)
      ex.hospital (in=va keep = hosp_id hospname
                   rename = (hosp_id=hosp)
                   where = (hospname eq: 'Veteran'));
by hosp ;
if adm and va;
run;

proc sort data=vahosp1d;
by pt_id admdate;
run;

title 'VAHOSP1D';
proc print data=vahosp1d;
run;

proc sql ;
create table vahosp2d as
select * from ex.admits
where hosp eq (select hosp_id 
               from ex.hospital 
               where hospname like "Veteran%")
order by pt_id, admdate ;
quit;

title 'VAHOSP2D';
proc print data=vahosp2d;
run;

* EXAMPLE 3
count admissions, min/max length of stay for each patient with at least on admission
;

data admsum1 ;
set ex.admits ;
by pt_id;

** (1) initialization;
if first.pt_id then do;
nstays = 0;
minlos = .;
maxlos = .;
end;

** (2) accumulation;
nstays = nstays + 1;
los = (disdate - admdate) + 1;
minlos = min(of minlos los) ;
maxlos = max(of maxlos los) ;

** (3) output;
if last.pt_id then output ;

retain nstays minlos maxlos ;
keep pt_id nstays minlos maxlos ;
run;

title 'ADMSUM1';
proc print data=admsum1;
run;

* EXTRA
ues a data set view plus a procedure
;

data newlos / view=newlos;
set ex.admits;
los = (disdate - admdate) + 1;
run;

proc means data=newlos maxdec=0 min max;
var los;
class pt_id;
run;

proc sql;
create table admsum2 as
select pt_id, count(*) as nstays,
       min(disdate - admdate + 1) as minlos,
       max(disdate - admdate + 1) as maxlos
from ex.admits
group by pt_id ;
quit;

title 'ADMSUM2';
proc print data=admsum2;
run;

* EXTRA
add variable los  to the data set --- what happens?
;

proc sql;
create table admsum2a2 as
select pt_id, count(*) as nstays, (disdate - admdate + 1) as los,
       min(calculated los) as minlos,
       max(calculated los) as maxlos
from ex.admits
group by pt_id ;
quit;

title 'ADMSUM2A2';
proc print data=admsum2a2;
run;

* EXAMPLE 4A
select observations based on summary functions
blood pressure outliers - ge +/- two standard deviations from the data set mean 
;

proc summary data= ex.admits ;
var bp_sys ;
output out=bpstats mean(bp_sys)=mean_sys std(bp_sys)=sd_sys ;
run;

data hi_sys1 ;
set bpstats (keep=mean_sys sd_sys)
    ex.admits ;
if _n_ eq 1 then do;
   high = mean_sys + 2*(sd_sys) ;
   low  = mean_sys - 2*(sd_sys) ;
   delete;
end;
retain high low;
if (bp_sys ge high) or (bp_sys le low) ;
drop mean_sys sd_sys high low ;
run;

title 'HI_SYS1';
proc print data=hi_sys1;
run;

proc sql ;
create table hi_sys2 as
select * from ex.admits
   where (bp_sys ge (select (mean(bp_sys) + 2*std(bp_sys))
   from ex.admits))
   or    (bp_sys le (select (mean(bp_sys) - 2*std(bp_sys))
   from ex.admits));
quit;

title 'HI_SYS2';
proc print data=hi_sys2;
run;


* EXAMPLE 4B
select observations based on summary functions
blood pressure outliers - ge +/- two standard deviations from the discharge destination mean 
a "remerge" since statistics are calculated by discharge destination
;

proc summary data= ex.admits nway;
class dest ;
var bp_sys ;
output out=bpstats2 mean(bp_sys)=mean_sys std(bp_sys)=sd_sys ;
run;

proc sort data = ex.admits out=admits;
by dest ;
run;

data hi_sys3 ;
merge admits (keep = pt_id bp_sys bp_dia dest)
      bpstats2 (keep = dest mean_sys sd_sys);
by dest ;
if bp_sys ge mean_sys + 2*(sd_sys) or
   bp_sys le mean_sys - 2*(sd_sys) ;
format mean_sys sd_sys 6.2;
run;

title 'HI_SYS3';
proc print data=hi_sys3;
run;

* alternative data set method;

data hi_sys3;
set admits (keep = pt_id bp_sys bp_dia dest);
by dest;
if first.dest then set bpstats2 (keep = dest mean_sys sd_sys); 
if bp_sys ge mean_sys + 2*(sd_sys) or
   bp_sys le mean_sys - 2*(sd_sys) ;
format mean_sys sd_sys 6.2;
run;

title 'HI_SYS3 - ALTERNATIVE DATA STEP METHOD';
proc print data=hi_sys3;
run;

proc sql;
create table hi_sys4 as
select pt_id, bp_sys, bp_dia, dest,
       mean(bp_sys) as mean_sys format=6.2,
       std(bp_sys)  as sd_sys   format=6.2
from ex.admits
group by dest
having bp_sys ge (mean(bp_sys) + 2*std(bp_sys))
    or bp_sys le (mean(bp_sys) - 2*std(bp_sys)) ;
quit;

title 'HI_SYS4';
proc print data=hi_sys4;
run;

* EXAMPLE 4C
identify duplicates
;

data selmd1 ;
set ex.doctors (keep   = md_id lastname hospadm
                rename = (hospadm=hospital));
by md_id ;
if not (first.md_id and last.md_id) ;
run;

proc sort data=selmd1 ;
by lastname hospital;
run;

title 'SELMD1';
proc print data=selmd1;
run;

proc sql ;
create table selmd2 as
select md_id, lastname, hospadm as hospital
from ex.doctors
group by md_id
having count(*) ge 2
order by lastname, hospital;
quit;

title 'SELMD2';
proc print data=selmd2;
run;


* EXAMPLE 5A
create two data sets from one (admits in 1997 and 1998)
;

data admits97 admits98;
set ex.admits;
if year(admdate) = 1997 theh output admits97;
else
if year(admdate) = 1998 then output admits98;
run;

title 'ADMITS97';
proc print data=admits97;
run;

title 'ADMITS98';
proc print data=admits98;
run;

proc sql;
create table adm97 as
   select * from ex.admits 
      where year(admdate) = 1997;
create table adm98 as
   select * from ex.admits 
      where year(admdate) = 1998;
quit; 

title 'ADM97';
proc print data=adm97;
run;

title 'ADM98';
proc print data=adm98;
run;

* EXAMPLE 5B
conctenate two data sets and create a third data set in patient id order
;

data alladm1;
set admits97 admits98;
by pt_id;
run;

title 'ALLADM1';
proc print data=alladm1;
run;

proc sql;
create table alladm2 as
   select * from adm97
      union
   select * from adm98
order by pt_id;
quit;

title 'ALLADM2';
proc print data=alladm2;
run;

* EXAMPLE 6A
combine observations  from two tables based on patient id number
;

* 
merge assumes data sets are sorted by pt_id and id
default output is all matched + unmatched observations from both data sets
restrict to only matched observations
;

data admits1 ;
merge ex.admits   (in=adm
                   keep=pt_id admdate disdate hosp md)
      ex.patients (in=pts
                   keep=id lastname sex primmd
                   rename=(id=pt_id));
by pt_id ;

if adm and pts;
run;

title 'ADMITS1';
proc print data=admits1;
run;

*
sql inner join makes no ssumptions as to sorted data sets
default output is matched observations, no unmatched
;
 
proc sql ;
create table admits2 as
select pt_id, admdate, disdate, hosp, md, lastname, sex, primmd
from ex.admits as a, ex.patients as b
where a.pt_id = b.id
order by a.pt_id, admdate;
quit;

title 'ADMITS2';
proc print data=admits2;
run;

*
alternate sql code - specify inner join
;

proc sql ;
create table admits2i as
select pt_id, admdate, disdate, hosp, md, lastname, sex, primmd
from ex.admits inner join ex.patients
on pt_id = id
order by pt_id, admdate ;
quit;

title 'ADMITS2I';
proc print data=admits2i;
run;

*
alternate sql code - speficy full join to produce default behavior of merge
;

proc sql ;
create table admits2f as
select pt_id, admdate, disdate, hosp, md, lastname, sex, primmd
from ex.admits full join ex.patients
on pt_id = id
order by pt_id, admdate ;
quit;

title 'ADMITS2F';
proc print data=admits2f;
run;

* EXAMPLE 6B
combine observations from three data sets
select only deaths
keep selcted variables from all three data set
;

* find patients who died;

data died1 (rename = (disdate=dthdate)) ;
merge ex.admits (in=dth
                 keep=pt_id disdate hosp dest
                 where=(dest='9'))
      ex.patients (in=pts
                   keep=id birthdte
                   rename=(id=pt_id));
by pt_id;
if dth and pts ;
agedth = floor((disdate - birthdte)/365.25) ;
drop dest birthdte ;
run;

proc sort data=died1;
by hosp;
run;

* add hospital information;

data died1b ;
merge died1 (in=dth
             rename=(hosp=hosp_id))
      ex.hospital (in=hsp
                   keep=hosp_id nbeds);
by hosp_id ;
if dth and hsp ;
drop hosp_id;
run;

proc sort data=died1b;
by pt_id ;
run;

title 'DIED1B';
proc print data=died1b;
run;

proc sql ;
create table died2 as
select pt_id, nbeds, disdate as dthdate,
       int((disdate-birthdte)/365.25) as agedth
from ex.admits, ex.hospital, ex.patients
where (pt_id = id) and (hosp = hosp_id) and dest eq '9'
order by pt_id ;
quit;

title 'DIED2';
proc print data=died2;
run;


* EXAMPLE 6C
create a data set with an observation for each hospital
show if a hospital had any admissions (variable hasadmit: 0/no, 1/yes)
;

proc sort data = ex.admits (keep = hosp)
          out=admits (rename=(hosp=hosp_id))
          nodupkey;
by hosp ;
run;

data hosps1 ;
merge ex.hospital (in=hosp)
      admits (in=adm);
by hosp_id ;
if hosp ;
hasadmit = adm ;
run;

title 'HOSPS1';
proc print data=hosps1;
run;

proc sql ;
create table hosps2 as
select distinct a.*,
       hosp is not null as hasadmit
from ex.hospital a left join ex.admits b
on a.hosp_id = b.hosp ;
quit;

title 'HOSPS2';
proc print data=hosps2;
run;


* EXAMPLE 6D
create data set with observations for patients treated their own MD
add MD name to the final data set
;

data prim1 (drop = primmd);
merge ex.admits (in=adm
                 keep=pt_id admdate disdate hosp md)
      ex.patients (in=pts
                   keep=id lastname primmd
                   rename=(id=pt_id));
by pt_id ;
if adm and pts and (md eq primmd) ;
run;

proc sort data=prim1;
by md;
run;

data doctors ;
set ex.doctors (keep=md_id lastname);
by md_id ;
if first.md_id ;
run;

data prim1a ;
merge prim1 (in=p
             rename=(lastname=ptname md=md_id))
      doctors (rename=(lastname=mdname));
by md_id ;
if p ;
run;

title 'PRIM1A';
proc print data=prim1a;
run;

proc sql ;
create table prim2 as
select pt_id, admdate, disdate, hosp, md_id,
       b.lastname as ptname,
       c.lastname as mdname
from ex.admits a, 
     ex.patients b,
     (select distinct md_id, lastname from ex.doctors) c
     where (a.pt_id eq b.id)  and
           (a.md eq b.primmd) and
           (a.md eq c.md_id)
order by a.pt_id, admdate ;
quit;

title 'PRIM2';
proc print data=prim2;
run;


* EXAMPLE 7
create a data set with MDs who had admissions to the VA hospital
;

proc sort data=ex.admits (keep=md hosp)
          out=admits;
by md;
run;

proc sort data=ex.doctors
          out=doctors
          nodupkey ;
by md_id ;
run;

data vadocs1 (drop = hosp);
merge doctors (in=docs
               keep=md_id lastname)
      admits  (in=adm
               where=(hosp = 3)
               rename = (md=md_id)) ;
by md_id;
if docs and adm and first.md_id ;
run;

proc sort;
by lastname;
run;

title 'VADOCS1';
proc print data=vadocs1;
run;

proc sql;
create table vadocs2 as
select distinct md_id, lastname
from ex.doctors as d
where 3 in (select hosp
            from ex.admits as a
            where d.md_id = a.md)
order by lastname;
quit;

title 'VADOCS2';
proc print data=vadocs2;
run;
